@@ -6,7 +6,7 @@ The ``decorator`` module
 :E-mail: michele.simionato@gmail.com
 :Version: $VERSION ($DATE)
 :Requires: Python 2.4+
-:Download page: http://pypi.python.org/pypi/decorator
+:Download page: http://pypi.python.org/pypi/decorator/$VERSION
 :Installation: ``easy_install decorator``
 :License: BSD license
 
@@ -275,16 +275,15 @@ For instance, you can write directly
  ...     print "calling %s with args %s, %s" % (f.func_name, args, kw)
  ...     return f(*args, **kw)
 
-and now ``trace`` will be a decorator. You
-can easily check that the signature has changed:
+and now ``trace`` will be a decorator. Actually ``trace`` is a ``partial``
+object which can be used as a decorator:
 
 .. code-block:: python
 
- >>> print getargspec(trace)
- (['f'], None, None, None)
+ >>> trace # doctest: +ELLIPSIS
+ <function trace at 0x...>
 
-Therefore now ``trace`` can be used as a decorator and
-the following will work:
+Here is an example of usage:
 
 .. code-block:: python
 
@@ -294,21 +293,62 @@ the following will work:
  >>> func()
  calling func with args (), {}
 
-For the rest of this document, I will discuss examples of useful
-decorators built on top of ``decorator``.
+If you are using an old Python version (Python 2.4) the
+``decorator`` module provides a poor man replacement for
+``functools.partial``.
+
+There is also an easy way to create one-parameter factories of
+decorators, based on the following
+``decorator_factory`` utility:
+
+$$decorator_factory
+
+``decorator_factory`` converts a function with signature
+``(param, func, *args, **kw)`` into a one-parameter family
+of decorators. Suppose for instance you want to generated different
+tracing generator, with different tracing messages.
+Here is how to do it:
+
+.. code-block:: python
+
+ >>> @decorator_factory
+ ... def trace_factory(message_template, f, *args, **kw):
+ ...     name = f.func_name
+ ...     print message_template % locals()
+ ...     return f(*args, **kw)
+
+.. code-block:: python
+
+ >>> trace_factory # doctest: +ELLIPSIS
+ <functools.partial object at 0x...>
+
+ >>> trace = trace_factory('Calling %(name)s with args %(args)s '
+ ...                        'and keywords %(kw)s')
+
+In this example the parameter (``message_template``) is 
+just a string, but in general it can be a tuple, a dictionary, or
+a generic object, so there is no real restriction (for instance,
+if you want to define a two-parameter family of decorators just
+use a tuple with two arguments as parameter).
+Here is an example of usage:
+
+.. code-block:: python
+
+ >>> @trace
+ ... def func(): pass
+
+ >>> func()
+ Calling func with args () and keywords {}
 
 ``blocking``
 -------------------------------------------
 
 Sometimes one has to deal with blocking resources, such as ``stdin``, and
 sometimes it is best to have back a "busy" message than to block everything. 
-This behavior can be implemented with a suitable decorator:
+This behavior can be implemented with a suitable family of decorators,
+where the parameter is the busy message:
 
 $$blocking
-
-(notice that without the help of ``decorator``, an additional level of 
-nesting would have been needed). This is actually an example
-of a one-parameter family of decorators.
    
 Functions decorated with ``blocking`` will return a busy message if
 the resource is unavailable, and the intended result if the resource is 
@@ -387,13 +427,13 @@ be locked. Here is a minimalistic example:
 Each call to ``write`` will create a new writer thread, but there will 
 be no synchronization problems since ``write`` is locked.
 
->>> write("data1") 
-<Thread(write-1, started)>
+>>> write("data1") # doctest: +ELLIPSIS
+<Thread(write-1, started...)>
 
 >>> time.sleep(.1) # wait a bit, so we are sure data2 is written after data1
 
->>> write("data2") 
-<Thread(write-2, started)>
+>>> write("data2") # doctest: +ELLIPSIS
+<Thread(write-2, started...)>
 
 >>> time.sleep(2) # wait for the writers to complete
 
@@ -409,9 +449,62 @@ a ``FunctionMaker`` class which is able to generate on the fly
 functions with a given name and signature from a function template
 passed as a string. Generally speaking, you should not need to
 resort to ``FunctionMaker`` when writing ordinary decorators, but
-it is handy in some circumstances. We will see an example in two
-paragraphs, when implementing a custom decorator factory
-(``decorator_apply``).
+it is handy in some circumstances. You will see an example shortly, in
+the implementation of a cool decorator utility (``decorator_apply``).
+
+``FunctionMaker`` provides a ``.create`` classmethod which
+takes as input the name, signature, and body of the function
+we want to generate as well as the execution environment
+were the function is generated by ``exec``. Here is an example:
+
+.. code-block:: python
+
+ >>> def f(*args, **kw): # a function with a generic signature
+ ...     print args, kw
+
+ >>> f1 = FunctionMaker.create('f1(a, b)', 'f(a, b)', dict(f=f))
+ >>> f1(1,2)
+ (1, 2) {}
+
+It is important to notice that the function body is interpolated
+before being executed, so be careful with the ``%`` sign!
+
+``FunctionMaker.create`` also accepts keyword arguments and such
+arguments are attached to the resulting function. This is useful
+if you want to set some function attributes, for instance the
+docstring ``__doc__``.
+
+For debugging/introspection purposes it may be useful to see
+the source code of the generated function; to do that, just
+pass the flag ``addsource=True`` and a ``__source__`` attribute will
+be added to the generated function:
+
+.. code-block:: python
+
+ >>> f1 = FunctionMaker.create(
+ ...     'f1(a, b)', 'f(a, b)', dict(f=f), addsource=True)
+ >>> print f1.__source__
+ def f1(a, b):
+     f(a, b)
+ <BLANKLINE>
+
+``FunctionMaker.create`` can take as first argument a string,
+as in the examples before, or a function. This is the most common
+usage, since typically you want to decorate a pre-existing
+function. A framework author may want to use directly ``FunctionMaker.create``
+instead of ``decorator``, since it gives you direct access to the body
+of the generated function. For instance, suppose you want to instrument
+the ``__init__`` methods of a set of classes, by preserving their
+signature (such use case is not made up; this is done in SQAlchemy
+and in other frameworks). When the first argument of ``FunctionMaker.create``
+is a function, a ``FunctionMaker`` object is instantiated internally,
+with attributes ``args``, ``varargs``,
+``keywords`` and ``defaults`` which are the
+the return values of the standard library function ``inspect.getargspec``.
+For each argument in the ``args`` (which is a list of strings containing
+the names of the mandatory arguments) an attribute ``arg0``, ``arg1``,
+..., ``argN`` is also generated. Finally, there is a ``signature`` 
+attribute, a string with the signature of the original function.
 
 Notice that while I do not have plans
 to change or remove the functionality provided in the
@@ -424,57 +517,10 @@ On the other hand, the functionality provided by
 ``decorator`` has been there from version 0.1 and it is guaranteed to
 stay there forever.
 
-``FunctionMaker`` takes the name and the signature (as a string) of a
-function in input, or a whole function. Then, it creates a new
-function (actually a closure) from a function template (the function
-template must begin with ``def`` with no comments before and you cannot
-use a ``lambda``) via its
-``.make`` method: the name and the signature of the resulting function
-are determinated by the specified name and signature.  For instance,
-here is an example of how to restrict the signature of a function:
-
-.. code-block:: python
-
- >>> def f(*args, **kw): # a function with a generic signature
- ...     print args, kw
-
- >>> fun = FunctionMaker(name="f1", signature="a,b")
- >>> f1 = fun.make('''\
- ... def %(name)s(%(signature)s):
- ...     f(%(signature)s)''', dict(f=f))
- ...
- >>> f1(1,2)
- (1, 2) {}
-
-The dictionary passed in this example (``dict(f=f)``) is the
-execution environment: ``FunctionMaker.make`` actually returns a
-closure, and the original function ``f`` is a variable in the
-closure environment.
-``FunctionMaker.make`` also accepts keyword arguments and such
-arguments are attached to the resulting function. This is useful
-if you want to set some function attributes, for instance the
-docstring ``__doc__``.
-
-For debugging/introspection purposes it may be useful to see
-the source code of the generated function; to do that, just
-pass the flag ``addsource=True`` and a ``__source__`` attribute will
-be added to the decorated function:
-
-.. code-block:: python
-
- >>> f1 = fun.make('''\
- ... def %(name)s(%(signature)s):
- ...     f(%(signature)s)''', dict(f=f), addsource=True)
- ...
- >>> print f1.__source__
- def f1(a,b):
-     f(a,b)
- <BLANKLINE>
-
 Getting the source code
 ---------------------------------------------------
 
-Internally ``FunctionMaker.make`` uses ``exec`` to generate the
+Internally ``FunctionMaker.create`` uses ``exec`` to generate the
 decorated function. Therefore
 ``inspect.getsource`` will not work for decorated functions. That
 means that the usual '??' trick in IPython will give you the (right on
@@ -491,7 +537,7 @@ $$identity_dec
  @identity_dec
  def example(): pass
 
- >>> print getsource(example)
+ >>> print inspect.getsource(example)
      def wrapper(*args, **kw):
          return func(*args, **kw)
  <BLANKLINE>
@@ -506,7 +552,7 @@ undecorated function:
 
 .. code-block:: python
 
- >>> print getsource(factorial.undecorated)
+ >>> print inspect.getsource(factorial.undecorated)
  @tail_recursive
  def factorial(n, acc=1):
      "The good old factorial"
@@ -625,13 +671,13 @@ function is decorated the traceback will be longer:
 .. code-block:: python
 
  >>> f()
- calling f with args (), {}
  Traceback (most recent call last):
-   File "<stdin>", line 1, in <module>
-   File "<string>", line 2, in f
-   File "documentation.py", line 799, in _trace
-     return f(*args, **kw)
-   File "<stdin>", line 3, in f
+   ...
+      File "<string>", line 2, in f
+      File "<doctest __main__[18]>", line 4, in trace
+        return f(*args, **kw)
+      File "<doctest __main__[47]>", line 3, in f
+        1/0
  ZeroDivisionError: integer division or modulo by zero
 
 You see here the inner call to the decorator ``trace``, which calls 
@@ -647,17 +693,49 @@ would require to change the CPython implementation of functions and
 add an hook to make it possible to change their signature directly. 
 That could happen in future versions of Python (see PEP 362_) and 
 then the decorator module would become obsolete. However, at present,
-even in Python 3.0 it is impossible to change the function signature
+even in Python 3.1 it is impossible to change the function signature
 directly, therefore the ``decorator`` module is still useful.
-Actually, this is one of the main reasons why I am releasing version 3.0.
+Actually, this is one of the main reasons why I keep maintaining
+the module and releasing new versions.
 
 .. _362: http://www.python.org/dev/peps/pep-0362
 
 In the present implementation, decorators generated by ``decorator``
 can only be used on user-defined Python functions or methods, not on generic 
 callable objects, nor on built-in functions, due to limitations of the
-``inspect`` module in the standard library.
-    
+``inspect`` module in the standard library. Moreover, notice
+that you can decorate a method, but only before if becomes a bound or unbound
+method, i.e. inside the class.
+Here is an example of valid decoration:
+
+.. code-block:: python
+
+ >>> class C(object): 
+ ...      @trace
+ ...      def meth(self):
+ ...          pass
+
+Here is an example of invalid decoration, when the decorator in
+called too late:
+
+.. code-block:: python
+
+ >>> class C(object):
+ ...      def meth(self):
+ ...          pass
+ ...
+ >>> trace(C.meth)
+ Traceback (most recent call last):
+   ...
+ TypeError: You are decorating a non function: <unbound method C.meth>
+
+The solution is to extract the inner function from the unbound method:
+
+.. code-block:: python
+
+ >>> trace(C.meth.im_func) # doctest: +ELLIPSIS
+ <function meth at 0x...>
+
 There is a restriction on the names of the arguments: for instance,
 if try to call an argument ``_call_`` or ``_func_``
 you will get a ``NameError``:
@@ -716,6 +794,9 @@ Finally ``decorator`` cannot be used as a class decorator and the
 `functionality introduced in version 2.3`_ has been removed. That
 means that in order to define decorator factories with classes you
 need to define the ``__call__`` method explicitly (no magic anymore).
+Since there is
+an easy way to define decorator factories by using ``decorator_factory``,
+there is less need to use classes to implement decorator factories.
 
 All these changes should not cause any trouble, since they were
 all rarely used features. Should you have any trouble, you can always
@@ -725,10 +806,9 @@ The examples shown here have been tested with Python 2.5. Python 2.4
 is also supported - of course the examples requiring the ``with``
 statement will not work there. Python 2.6 works fine, but if you
 run the examples here in the interactive interpreter
-you will notice a couple of minor differences since
+you will notice a few differences since
 ``getargspec`` returns an ``ArgSpec`` namedtuple instead of a regular
-tuple, and the string representation of a thread object returns a
-thread identifier number. That means that running the file
+tuple. That means that running the file
 ``documentation.py`` under Python 2.5 will a few errors, but
 they are not serious. Python 3.0 is kind of supported too.
 Simply run the script ``2to3`` on the module
@@ -749,6 +829,9 @@ Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met::
 
+  Copyright (c) 2005, Michele Simionato
+  All rights reserved.
+
   Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.
   Redistributions in bytecode form must reproduce the above copyright
@@ -775,23 +858,26 @@ you are unhappy with it, send me a patch!
 """
 from __future__ import with_statement
 import sys, threading, time, functools, inspect, itertools  
-try:
-    import multiprocessing
-except ImportError:
-    import processing as multiprocessing
 from decorator import *
+from functools import partial
 from setup import VERSION
 
 today = time.strftime('%Y-%m-%d')
 
 __doc__ = __doc__.replace('$VERSION', VERSION).replace('$DATE', today)
 
+def decorator_factory(decfac): # partial is functools.partial
+    "decorator_factory(decfac) returns a one-parameter family of decorators"
+    return partial(lambda df, param: decorator(partial(df, param)), decfac)
+
 def decorator_apply(dec, func):
-    "Decorate a function using a signature-non-preserving decorator"
-    fun = FunctionMaker(func)
-    src = '''def %(name)s(%(signature)s):
-    return decorated(%(signature)s)'''
-    return fun.make(src, dict(decorated=dec(func)), undecorated=func)
+    """
+    Decorate a function by preserving the signature even if dec 
+    is not a signature-preserving decorator.
+    """
+    return FunctionMaker.create(
+        func, 'return decorated(%(signature)s)',
+        dict(decorated=dec(func)), undecorated=func)
 
 def _trace(f, *args, **kw):
     print "calling %s with args %s, %s" % (f.__name__, args, kw)
@@ -886,20 +972,19 @@ def _memoize(func, *args, **kw):
 def memoize(f):
     f.cache = {}
     return decorator(_memoize, f)
-
-def blocking(not_avail="Not Available"):
-    def _blocking(f, *args, **kw):
-        if not hasattr(f, "thread"): # no thread running
-            def set_result(): f.result = f(*args, **kw)
-            f.thread = threading.Thread(None, set_result)
-            f.thread.start()
-            return not_avail
-        elif f.thread.isAlive():
-            return not_avail
-        else: # the thread is ended, return the stored result
-            del f.thread 
-            return f.result
-    return decorator(_blocking)
+    
+@decorator_factory
+def blocking(not_avail, f, *args, **kw):
+    if not hasattr(f, "thread"): # no thread running
+        def set_result(): f.result = f(*args, **kw)
+        f.thread = threading.Thread(None, set_result)
+        f.thread.start()
+        return not_avail
+    elif f.thread.isAlive():
+        return not_avail
+    else: # the thread is ended, return the stored result
+        del f.thread
+        return f.result
 
 class User(object):
     "Will just be able to see a page"
@@ -916,73 +1001,68 @@ def get_userclass():
 class PermissionError(Exception):
     pass
 
-class Restricted(object):
-    """
-    Restrict public methods and functions to a given class of users.
-    If instantiated twice with the same userclass return the same
-    object.
-    """
-    _cache = {} 
-    def __new__(cls, userclass):
-        if userclass in cls._cache:
-            return cls._cache[userclass]
-        self = cls._cache[userclass] = super(Restricted, cls).__new__(cls)
-        self.userclass = userclass
-        return self
-    def call(self, func, *args, **kw):
-        userclass = get_userclass()
-        if issubclass(userclass, self.userclass):
-            return func(*args, **kw)
-        else:
-            raise PermissionError(
+@decorator_factory
+def restricted(user_class, func, *args, **kw):
+    "Restrict access to a given class of users"
+    userclass = get_userclass()
+    if issubclass(userclass, user_class):
+        return func(*args, **kw)
+    else:
+        raise PermissionError(
             '%s does not have the permission to run %s!'
             % (userclass.__name__, func.__name__))
-    def __call__(self, func):
-        return decorator(self.call, func)
 
 class Action(object):
-    @Restricted(User)
+    """
+    >>> a = Action()
+    >>> a.view() # ok
+    >>> a.insert() # err
+    Traceback (most recent call last):
+       ...
+    PermissionError: User does not have the permission to run insert!
+
+    """
+    @restricted(User)
     def view(self):
         pass
 
-    @Restricted(PowerUser)
+    @restricted(PowerUser)
     def insert(self):
         pass
 
-    @Restricted(Admin)
+    @restricted(Admin)
     def delete(self):
         pass
-    
+
 class TailRecursive(object):
     """
     tail_recursive decorator based on Kay Schluehr's recipe
     http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/496691
+    with improvements by me and George Sakkis.
     """
-    CONTINUE = object() # sentinel
 
     def __init__(self, func):
         self.func = func
         self.firstcall = True
+        self.CONTINUE = object() # sentinel
 
     def __call__(self, *args, **kwd):
-        try:
-            if self.firstcall: # start looping
-                self.firstcall = False
-                while True:            
-                    result = self.func(*args, **kwd)
-                    if result is self.CONTINUE: # update arguments
+        CONTINUE = self.CONTINUE
+        if self.firstcall:
+            func = self.func
+            self.firstcall = False
+            try:
+                while True:
+                    result = func(*args, **kwd)
+                    if result is CONTINUE: # update arguments
                         args, kwd = self.argskwd
                     else: # last call
-                        break
-            else: # return the arguments of the tail call
-                self.argskwd = args, kwd
-                return self.CONTINUE
-        except: # reset and re-raise
-            self.firstcall = True
-            raise
-        else: # reset and exit
-            self.firstcall = True 
-            return result
+                        return result
+            finally:
+                self.firstcall = True
+        else: # return the arguments of the tail call
+            self.argskwd = args, kwd
+            return CONTINUE
 
 def tail_recursive(func):
     return decorator_apply(TailRecursive, func)
@@ -997,5 +1077,20 @@ def fact(n): # this is not tail-recursive
     if n == 0: return 1
     return n * fact(n-1)
 
+
+def atest_for_pylons():
+    """
+    In version 3.1.0 decorator(caller) returned a nameless partial
+    object, thus breaking Pylons. That must not happen again.
+
+    >>> decorator(_memoize).__name__
+    '_memoize'
+
+    Here is another bug of version 3.1.1 to avoid:
+
+    >>> deprecated.__doc__
+    'A decorator for deprecated functions'
+    """
+
 if __name__ == '__main__':
     import doctest; doctest.testmod()
